# -*- coding: utf-8 -*-

from functools import update_wrapper
from django.shortcuts import render
from django.http import HttpResponseForbidden
from django.template.loader import render_to_string
from django.conf import settings
from ldap_inp import LdapClub


def forbidden(request):
    request.desactiver_widgets = True
    return HttpResponseForbidden(render_to_string("403.html", None, request=request))


def access_restricted(function=None):
    if function:
        return _access_decorator(function)
    return _access_decorator


class _access_decorator(object):
    def __init__(self, view_func):
        self.view_func = view_func
        self.func_name = view_func.__name__

        update_wrapper(self, view_func, updated=())
        for k in view_func.__dict__:
            if k not in self.__dict__:
                self.__dict__[k] = view_func.__dict__[k]

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return _access_decorator(view_func)

    def __call__(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return self.view_func(request, *args, **kwargs)
        request.desactiver_widgets = True
        return render(request, "auth/access_restricted.html", status=401)


def permission_needed(perm, function=None):
    if function:
        return _permission_decorator(perm, function)
    return lambda function: _permission_decorator(perm, function)


class _permission_decorator(object):
    def __init__(self, perm, view_func):
        self.view_func = view_func
        self.perm = perm

        update_wrapper(self, view_func, updated=())
        for k in view_func.__dict__:
            if k not in self.__dict__:
                self.__dict__[k] = view_func.__dict__[k]

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return _permission_decorator(view_func)

    def __call__(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            request.desactiver_widgets = True
            return render(request, "auth/access_restricted.html", status=401)

        if request.user.check_permission(request, self.perm):
            return self.view_func(request, *args, **kwargs)

        return forbidden(request)


def none_if_anonymous(function=None):
    if function:
        return _anonymous_decorator(function)
    return function


class _anonymous_decorator(object):
    def __init__(self, view_func):
        self.view_func = view_func

        update_wrapper(self, view_func, updated=())
        for k in view_func.__dict__:
            if k not in self.__dict__:
                self.__dict__[k] = view_func.__dict__[k]

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return _anonymous_decorator(view_func)

    def __call__(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return self.view_func(request, *args, **kwargs)
        return None


def addr_inp(addr):
    # Blocs d’adresses des écoles de l’INP
    allowed_addr = settings.ALLOWED_ADDRS
    return any([addr.startswith(b) for b in allowed_addr])


def access_inp(function=None):
    if function:
        return _access_inp(function)
    return _access_inp


class _access_inp(object):
    def __init__(self, view_func):
        self.view_func = view_func
        self.func_name = view_func.__name__

        update_wrapper(self, view_func, updated=())
        for k in view_func.__dict__:
            if k not in self.__dict__:
                self.__dict__[k] = view_func.__dict__[k]

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return _access_inp(view_func)

    def __call__(self, request, *args, **kwargs):
        if addr_inp(request.META["REMOTE_ADDR"]) or request.user.is_authenticated:
            return self.view_func(request, *args, **kwargs)

        if self.func_name != "widget":
            request.desactiver_widgets = True
            return render(request, "auth/access_restricted.html", status=401)
        else:
            # les widgets retournent du texte
            return None


def membre_club(request, club):
    return (
        LdapClub.objects.filter(cn=club + "-" + request.ecole.ident, memberUid=request.user.username)
        or request.user.is_superuser
        or (request.user.is_staff and request.user.ecole == request.ecole)
    )


def access_club(club, function=None):
    if function:
        return _access_club(club, function)
    return lambda function: _access_club(club, function)


class _access_club(object):
    def __init__(self, club, view_func):
        self.view_func = view_func
        self.club = club
        self.func_name = view_func.__name__

        update_wrapper(self, view_func, updated=())
        for k in view_func.__dict__:
            if k not in self.__dict__:
                self.__dict__[k] = view_func.__dict__[k]

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return _access_decorator(view_func)

    def __call__(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            request.desactiver_widgets = True
            return render(request, "auth/access_restricted.html", status=401)

        if membre_club(request, self.club):
            return self.view_func(request, *args, **kwargs)

        return forbidden(request)


def access_frappe(function=None):
    if function:
        return _access_frappe(function)
    return _access_frappe


class _access_frappe(object):
    def __init__(self, view_func):
        self.view_func = view_func
        self.func_name = view_func.__name__

        update_wrapper(self, view_func, updated=())
        for k in view_func.__dict__:
            if k not in self.__dict__:
                self.__dict__[k] = view_func.__dict__[k]

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return _access_frappe(view_func)

    def __call__(self, request, *args, **kwargs):
        if request.user.is_authenticated and request.user.get_ldap_user().inscritFrappe:
            return self.view_func(request, *args, **kwargs)

        if request.user.is_authenticated:
            return render(request, "frappe/index_non_inscrit.html", status=403)

        return render(request, "auth/access_restricted.html", status=401)


def access_club_or_permission(club, permission):
    def decorator(fonction):
        def wrap(request, *args, **kwargs):
            if not request.user.is_authenticated:
                request.desactiver_widgets = True
                return render(request, "auth/access_restricted.html", status=401)

            if club:
                if membre_club(request, club):
                    return fonction(request, *args, **kwargs)

            if permission:
                if request.user.check_permission(request, permission):
                    return fonction(request, *args, **kwargs)

            return forbidden(request)

        return wrap

    return decorator
